@@ -2,7 +2,7 @@
 /*
  * libspeechd.c - Shared library for easy acces to Speech Dispatcher functions
  *
- * Copyright (C) 2001, 2002, 2003 Brailcom, o.p.s.
+ * Copyright (C) 2001, 2002, 2003, 2006 Brailcom, o.p.s.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: libspeechd.c,v 1.27 2006/07/11 16:12:26 hanke Exp $
+ * $Id: libspeechd.c,v 1.29 2006/11/29 16:56:11 hanke Exp $
  */
 
 
@@ -130,7 +130,7 @@ spd_open(const char* client_name, const char* connection_name, const char* user_
     connection->callback_pause = NULL;
     connection->callback_resume = NULL;
     connection->callback_cancel = NULL;
-    
+
     connection->mode = mode;
 
     /* Create a stream from the socket */
@@ -160,25 +160,25 @@ spd_open(const char* client_name, const char* connection_name, const char* user_
 	    return NULL;
 	}
     }
-    
+
     setsockopt(connection->socket, IPPROTO_TCP, TCP_NODELAY, &tcp_no_delay, sizeof(int));
 
     /* By now, the connection is created and operational */
-    
+
     set_client_name = g_strdup_printf("SET SELF CLIENT_NAME \"%s:%s:%s\"", usr_name,
 				      client_name, conn_name);
-    
+
     ret = spd_execute_command_wo_mutex(connection, set_client_name);   
-    
+
     xfree(usr_name);  xfree(conn_name);  xfree(set_client_name);
-    
+
     return connection;
 }
 
 
 #define RET(r) \
     { \
-    pthread_mutex_lock(connection->ssip_mutex); \
+    pthread_mutex_unlock(connection->ssip_mutex); \
     return r; \
     }
 
@@ -407,7 +407,7 @@ spd_char(SPDConnection *connection, SPDPriority priority, const char *character)
     if (ret) RET(-1);
 
     sprintf(command, "CHAR %s", character);
-    ret = spd_execute_command(connection, command);
+    ret = spd_execute_command_wo_mutex(connection, command);
     if (ret) RET(-1);
 
     pthread_mutex_unlock(connection->ssip_mutex);
@@ -703,35 +703,12 @@ spd_set_notification(SPDConnection *connection, SPDNotification notification, co
 }
 #undef NOTIFICATION_SET
 
-int
-spd_get_client_list(SPDConnection *connection, char **client_names, int *client_ids, int* active){
-        SPD_DBG("spd_get_client_list: History is not yet implemented.");
-        return -1;
-#if 0
-	sprintf(command, "HISTORY GET CLIENT_LIST\r\n");
-	reply = (char*) spd_send_data(fd, command, 1);
-
-	footer_ok = parse_response_footer(reply);
-	if(footer_ok != 1){
-		free(reply);
-		return -1;
-	}
-	
-	for(count=0;  ;count++){
-		record = (char*) parse_response_data(reply, count+1);
-		if (record == NULL) break;
-		record_int = get_rec_int(record, 0);
-		client_ids[count] = record_int;
-		record_str = (char*) get_rec_str(record, 1);
-		assert(record_str!=NULL);
-		client_names[count] = record_str;
-		record_int = get_rec_int(record, 2);
-		active[count] = record_int;
-	}	
-	return count;
-
-#endif
-}
+//int
+//spd_get_client_list(SPDConnection *connection, char **client_names, int *client_ids, int* active){
+//        SPD_DBG("spd_get_client_list: History is not yet implemented.");
+//        return -1;
+//
+//}
 
 int
 spd_get_message_list_fd(SPDConnection *connection, int target, int *msg_ids, char **client_names)
