@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2006 Brailcom, o.p.s.
+# Copyright (C) 2003-2007 Brailcom, o.p.s.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,8 +23,41 @@ A more convenient interface is provided by the 'Speaker' class.
 
 """
 
-import socket
+#TODO: Blocking variants for speak, char, key, sound_icon.
 
+import socket
+import sys
+
+try:
+    import threading
+except:
+    import dummy_threading as threading
+
+    
+class CallbackType(object):
+    """Constants describing the available types of callbacks"""
+    INDEX_MARK = 'index_marks'
+    """Index mark events are reported when the place they were
+    included into the text by the client application is reached
+    when speaking them"""
+    BEGIN = 'begin'
+    """The begin event is reported when Speech Dispatcher starts
+    actually speaking the message."""
+    END = 'end'
+    """The end event is reported after the message has terminated and
+    there is no longer any sound from it being produced"""
+    CANCEL = 'cancel'
+    """The cancel event is reported when a message is canceled either
+    on request of the user, because of prioritization of messages or
+    due to an error"""
+    PAUSE = 'pause'
+    """The pause event is reported after speaking of a message
+    was paused. It no longer produces any audio."""
+    RESUME = 'resume'
+    """The resume event is reported right after speaking of a message
+    was resumed after previous pause."""
+
+    
 class SSIPError(Exception):
     def __init__(self, code, msg, data):
         Exception.__init__(self, "%s: %s" % (code, msg))
@@ -61,25 +94,82 @@ class _SSIP_Connection:
     """Implemantation of low level SSIP communication."""
     
     _NEWLINE = "\r\n"
-    """New line delimeter as a string."""
-
     _END_OF_DATA_MARKER = '.'
-    """Data end marker."""
-
     _END_OF_DATA_MARKER_ESCAPED = '..'
-    """Escaped data end marker."""
-    
     _END_OF_DATA = _NEWLINE + _END_OF_DATA_MARKER + _NEWLINE
-    """Data end marker."""
-
     _END_OF_DATA_ESCAPED = _NEWLINE + _END_OF_DATA_MARKER_ESCAPED + _NEWLINE
-    """Data may contain a marker string, so we need to escape it..."""
 
+    _CALLBACK_TYPE_MAP = {700: CallbackType.INDEX_MARK,
+                          701: CallbackType.BEGIN,
+                          702: CallbackType.END,
+                          703: CallbackType.CANCEL,
+                          704: CallbackType.PAUSE,
+                          705: CallbackType.RESUME,
+                          }
+    
     def __init__(self, host, port):
+        """Init connection: open the socket to server,
+        initialize buffers, launch a communication handling
+        thread."""
         self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self._socket.connect((socket.gethostbyname(host), port))
         self._buffer = ""
+        self._com_buffer = []
+        self._callback = None
+        self._ssip_reply_semaphore = threading.Semaphore(0)
+        self._communication_thread = \
+                threading.Thread(target=self._communication, kwargs={},
+                                 name="SSIP client communication thread")
+        self._communication_thread.start()
     
+
+    def close(self):
+        """Close the server connection, destroy the communication thread."""
+        # Read-write shutdown here is necessary,
+        # otherwise the socket.recv() function
+        # in the other thread won't return at last
+        # on some platforms
+        self._socket.shutdown(socket.SHUT_RDWR)
+        self._socket.close()
+        # Wait for the other thread to terminate
+        self._communication_thread.join()
+        
+    def _communication(self):
+        """Handle incomming socket communication.
+
+        Listens for all incomming communication on the socket, dispatches
+        events and puts all other replies into self._com_buffer list in the
+        already parsed form as (code, msg, data).  Each time a new item is
+        appended to the _com_buffer list, the corresponding semaphore
+        'self._ssip_reply_semaphore' is incremented.
+
+        This method is designed to run in a separate thread.  The thread can be
+        interrupted by closing the socket on which it is listening for
+        reading."""
+
+        while True:
+            try:
+                code, msg, data = self._recv_message()
+            except IOError:
+                # If the socket has been closed, exit the thread
+                sys.exit()
+            if code/100 != 7:
+                # This is not an index mark nor an event
+                self._com_buffer.append((code, msg, data))
+                self._ssip_reply_semaphore.release()
+                continue
+            # Ignore the event if no callback function has been registered.
+            if self._callback is not None:
+                type = self._CALLBACK_TYPE_MAP[code]
+                if type == CallbackType.INDEX_MARK:
+                    kwargs = {'index_mark': data[2]}
+                else:
+                    kwargs = {}
+                # Get message and client ID of the event
+                msg_id, client_id = map(int, data[:2])
+                self._callback(msg_id, client_id, type, **kwargs)
+                
+                
     def _readline(self):
         """Read one whole line from the socket.
 
@@ -88,17 +178,24 @@ class _SSIP_Connection:
         """
         pointer = self._buffer.find(self._NEWLINE)
         while pointer == -1:
-            self._buffer += self._socket.recv(1024)
+            try:
+                d = self._socket.recv(1024)
+            except:
+                raise IOError
+            if len(d) == 0:
+                raise IOError
+            self._buffer += d
             pointer = self._buffer.find(self._NEWLINE)
         line = self._buffer[:pointer]
         self._buffer = self._buffer[pointer+len(self._NEWLINE):]
         return line
-            
-    def _recv_response(self):
-        """Read server response and return the triplet (code, msg, data)."""
+
+    def _recv_message(self):
+        """Read server response or a callback
+        and return the triplet (code, msg, data)."""
         data = []
         c = None
-        while 1:
+        while True:
             line = self._readline()
             assert len(line) >= 4, "Malformed data received from server!"
             code, sep, text = line[:3], line[3], line[4:]
@@ -109,8 +206,17 @@ class _SSIP_Connection:
                 return int(code), msg, tuple(data)
             data.append(text)
 
+    def _recv_response(self):
+        """Read server response from the communication thread
+        and return the triplet (code, msg, data)."""
+        self._ssip_reply_semaphore.acquire()
+        # The list is sorted, read the first item
+        response = self._com_buffer[0]
+        del self._com_buffer[0]
+        return response
+
     def send_command(self, command, *args):
-        """Send SSIP command with given arguments and read server responsse.
+        """Send SSIP command with given arguments and read server response.
 
         Arguments can be of any data type -- they are all stringified before
         being sent to the server.
@@ -127,8 +233,8 @@ class _SSIP_Connection:
         """
         if __debug__:
             if command in ('SET', 'CANCEL', 'STOP',):
-                assert args[0] in (Scope.SELF, Scope.ALL), scope
-                #or isinstance(args[0], int)
+                assert args[0] in (Scope.SELF, Scope.ALL) \
+                       or isinstance(args[0], int)
         cmd = ' '.join((command,) + tuple(map(str, args)))
         self._socket.send(cmd + self._NEWLINE)
         code, msg, data = self._recv_response()
@@ -137,7 +243,7 @@ class _SSIP_Connection:
         return code, msg, data
         
     def send_data(self, data):
-        """Send multiline data and read server responsse.
+        """Send multiline data and read server response.
 
         Returned value is the same as for 'send_command()' method.
 
@@ -154,17 +260,35 @@ class _SSIP_Connection:
             data = self._END_OF_DATA_MARKER_ESCAPED
         data = data.replace(self._END_OF_DATA, self._END_OF_DATA_ESCAPED)
         self._socket.send(data + self._END_OF_DATA)
-        code, msg, data = self._recv_response()
+        code, msg, response_data = self._recv_response()
         if code/100 != 2:
             raise SSIPDataError(code, msg, data)
-        return code, msg
-        
-    def close(self):
-        """Close the connection."""
-        self._socket.close()
-        self._socket = None
-            
+        return code, msg, response_data
 
+    def set_callback(self, callback):
+        """Register a callback function for handling asynchronous events.
+
+        Arguments:
+
+          callback -- a callable object (function) which will be called to
+            handle asynchronous events (arguments described below).  Passing
+            `None' results in removing the callback function and ignoring
+            events.  Just one callback may be registered.  Attempts to register
+            a second callback will result in the former callback being
+            replaced.
+
+        The callback function must accept three positional arguments
+        ('message_id', 'client_id', 'event_type') and an optional keyword
+        argument 'index_mark' (when INDEX_MARK events are turned on).
+
+        Note, that setting the callback function doesn't turn the events on.
+        The user is responsible to turn them on by sending the appropriate `SET
+        NOTIFICATION' command.
+
+        """
+        self._callback = callback
+
+            
 class Scope(object):
     """An enumeration of valid SSIP command scopes.
 
@@ -210,7 +334,6 @@ class PunctuationMode(object):
 
     """
 
-    
 class SSIPClient(object):
     """Basic Speech Dispatcher client interface.
 
@@ -257,7 +380,43 @@ class SSIPClient(object):
         self._conn = _SSIP_Connection(host, port or self.SPEECH_PORT)
         full_name = '%s:%s:%s' % (user, name, component)
         self._conn.send_command('SET', Scope.SELF, 'CLIENT_NAME', full_name)
+        result = self._conn.send_command('HISTORY', 'GET', 'CLIENT_ID')
+        self._client_id = int(result[2][0])
+        self._lock = threading.Lock()
+        self._callbacks = {}
+        self._conn.set_callback(self._callback_handler)
+        for event in (CallbackType.INDEX_MARK,
+                      CallbackType.BEGIN,
+                      CallbackType.END,
+                      CallbackType.CANCEL,
+                      CallbackType.PAUSE,
+                      CallbackType.RESUME):
+            self._conn.send_command('SET', 'self', 'NOTIFICATION', event, 'on')
 
+    
+    def __del__(self):
+        """Close the connection"""
+        self.close()
+
+    def _callback_handler(self, msg_id, client_id, type, **kwargs):
+        if client_id != self._client_id:
+            # TODO: does that ever happen?
+            return
+        self._lock.acquire()
+        try:
+            try:
+                callback, event_types = self._callbacks[msg_id]
+            except KeyError:
+                pass
+            else:
+                if event_types is None or type in event_types:
+                    callback(type, **kwargs)
+                if type in (CallbackType.END, CallbackType.CANCEL):
+                    del self._callbacks[msg_id]
+        finally:
+            self._lock.release()
+                
+        
     def set_priority(self, priority):
         """Set the priority category for the following messages.
 
@@ -271,31 +430,68 @@ class SSIPClient(object):
                             Priority.PROGRESS), priority
         self._conn.send_command('SET', Scope.SELF, 'PRIORITY', priority)
 
-    def speak(self, text):
+    def speak(self, text, callback=None, event_types=None):
         """Say given message.
 
         Arguments:
 
-          text -- message text to be spoken as string.
-          
+          text -- message text to be spoken.  This may be either a UTF-8
+            encoded byte string or a Python unicode string.
+
+          callback -- a callback handler for asynchronous event notifications.
+            A callable object (function) which accepts one positional argument
+            `type' and one keyword argument `index_mark'.  See below for more
+            details.
+
+          event_types -- a tuple of event types for which the callback should
+            be called.  Each item must be one of `CallbackType' constants.
+            None (the default value) means to handle all event types.  This
+            argument is irrelevant when `callback' is not used.
+
+        The callback function will be called whenever one of the events occurs.
+        The event type will be passed as argument.  Its value is one of the
+        `CallbackType' constants.  In case of an index mark event, additional
+        keyword argument `index_mark' will be passed and will contain the index
+        mark identifier as specified within the text.
+
+        The callback function should not perform anything complicated and is
+        not allowed to issue any further SSIP client commands.  An attempt to
+        do so would lead to a deadlock in SSIP communication.
+
         This method is non-blocking;  it just sends the command, given
         message is queued on the server and the method returns immediately.
 
         """
         self._conn.send_command('SPEAK')
-        self._conn.send_data(text)
+        if isinstance(text, unicode):
+            text = text.encode('utf-8')
+        result = self._conn.send_data(text)
+        if callback:
+            msg_id = int(result[2][0])
+            # TODO: Here we risk, that the callback arrives earlier, than we
+            # add the item to `self._callbacks'.  Such a situation will lead to
+            # the callback being ignored.
+            self._lock.acquire()
+            try:
+                self._callbacks[msg_id] = (callback, event_types)
+            finally:
+                self._lock.release()
+        return result
 
     def char(self, char):
         """Say given character.
 
         Arguments:
 
-          char -- a character to be spoken (as a unicode string of length 1).
+          char -- a character to be spoken.  Either a Python unicode string or
+            a UTF-8 encoded byte string.
 
         This method is non-blocking;  it just sends the command, given
         message is queued on the server and the method returns immediately.
 
         """
+        if isinstance(char, unicode):
+            char = char.encode('utf-8')
         self._conn.send_command('CHAR', char.replace(' ', 'space'))
         
     def key(self, key):
@@ -303,7 +499,7 @@ class SSIPClient(object):
 
         Arguments:
 
-          key -- the key name (as defined in SSIP).
+          key -- the key name (as defined in SSIP); string.
 
         This method is non-blocking;  it just sends the command, given
         message is queued on the server and the method returns immediately.
@@ -316,7 +512,7 @@ class SSIPClient(object):
 
         Arguments:
 
-          sound_icon -- the name of the sound icon as defined by SSIP.
+          sound_icon -- the name of the sound icon as defined by SSIP; string.
 
         This method is non-blocking; it just sends the command, given message
         is queued on the server and the method returns immediately.
@@ -531,10 +727,11 @@ class SSIPClient(object):
 
         """
         self._conn.send_command('BLOCK', 'END')
-        
+
     def close(self):
         """Close the connection to Speech Dispatcher."""
-        self._conn.close()
+        if hasattr(self, '_conn'):
+            self._conn.close()
 
 
 class Client(SSIPClient):
@@ -582,4 +779,5 @@ class Speaker(SSIPClient):
     Well, in fact this class is currently not implemented at all.  It is just a
     draft.  The intention is to hide the SSIP details and provide a generic
     interface practical for screen readers.
+    
     """
