@@ -16,10 +16,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
- * $Id: set.c,v 1.39 2006/01/08 13:36:58 hanke Exp $
+ * $Id: set.c,v 1.41 2006/07/11 16:12:27 hanke Exp $
  */
 
 #include <fnmatch.h>
@@ -417,6 +417,7 @@ default_fd_set(void)
 	new->hist_cur_pos = -1;
 	new->hist_sorted = 0;
 	new->index_mark = NULL;
+	new->paused_while_speaking = 0;
 
 	return(new);
 }
