@@ -16,10 +16,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
- * $Id: run_test.c,v 1.10 2006/01/08 13:36:58 hanke Exp $
+ * $Id: run_test.c,v 1.13 2006/07/11 16:12:28 hanke Exp $
  */
 
 #include <stdio.h>
@@ -62,6 +62,28 @@ send_data(int fd, char *message, int wfr)
 	return reply;
 }
 
+void
+wait_for(int fd, char* event)
+{
+    char * reply;
+    int bytes;
+
+    printf("       Waiting for: |%s|\n", event);
+    reply = (char*) malloc(sizeof(char) * 1000);
+    reply[0] = 0;
+    while (NULL == strcasestr(reply, event)) {
+        bytes = read(fd, reply, 1000);
+        if (bytes > 0) {
+            reply[bytes] = 0;
+            printf("       < %s\n", reply);
+            fflush(NULL);
+        }
+    }
+    free(reply);
+    printf("       Continuing.\n", reply);
+    fflush(NULL);
+}
+
 int 
 init(char* client_name, char* conn_name)
 {
@@ -167,6 +189,12 @@ main(int argc, char* argv[])
             continue;
         }
 
+        if(line[0] == '+'){
+            command = (char*) strtok(&(line[1]), "+\r\n");
+            wait_for(sockk, command);
+            continue;
+        }
+
         if(line[0] == '$'){
             if (delays){
                 command = (char*) strtok(&(line[1]), "$\r\n");
